"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConstantProductPool = void 0;
const all_json_1 = __importDefault(require("@sushiswap/trident/exports/all.json"));
const soulswap_currency_1 = require("soulswap-currency");
const soulswap_math_1 = require("soulswap-math");
const tiny_invariant_1 = __importDefault(require("tiny-invariant"));
const errors_1 = require("../errors");
const computeConstantProductPoolAddress_1 = require("./computeConstantProductPoolAddress");
class ConstantProductPool {
    constructor(AmountA, AmountB, fee, twap) {
        this.swapGasCost = soulswap_math_1.JSBI.BigInt(60000);
        this.minLiquidity = soulswap_math_1.JSBI.BigInt(1000);
        this.MAX_FEE = soulswap_math_1.JSBI.BigInt(10000);
        const Amounts = AmountA.currency.sortsBefore(AmountB.currency) // does safety checks
            ? [AmountA, AmountB]
            : [AmountB, AmountA];
        this.liquidityToken = new soulswap_currency_1.Token({
            chainId: Amounts[0].currency.chainId,
            address: ConstantProductPool.getAddress(Amounts[0].currency, Amounts[1].currency, fee, twap),
            decimals: 18,
            symbol: 'SCPLP',
            name: 'Sushi Constant Product LP Token',
        });
        this.fee = fee;
        this.twap = twap;
        this.tokenAmounts = Amounts;
    }
    static getAddress(tokenA, tokenB, fee, twap) {
        return (0, computeConstantProductPoolAddress_1.computeConstantProductPoolAddress)({
            factoryAddress: all_json_1.default[tokenA.chainId][0].contracts.ConstantProductPoolFactory.address,
            tokenA,
            tokenB,
            fee,
            twap,
        });
    }
    /**
     * Returns true if the token is either token0 or token1
     * @param token to check
     */
    involvesToken(token) {
        return token.equals(this.token0) || token.equals(this.token1);
    }
    /**
     * Returns the current mid price of the pair in terms of token0, i.e. the ratio of reserve1 to reserve0
     */
    get token0Price() {
        const result = this.tokenAmounts[1].divide(this.tokenAmounts[0]);
        return new soulswap_currency_1.Price(this.token0, this.token1, result.denominator, result.numerator);
    }
    /**
     * Returns the current mid price of the pair in terms of token1, i.e. the ratio of reserve0 to reserve1
     */
    get token1Price() {
        const result = this.tokenAmounts[0].divide(this.tokenAmounts[1]);
        return new soulswap_currency_1.Price(this.token1, this.token0, result.denominator, result.numerator);
    }
    /**
     * Return the price of the given token in terms of the other token in the pair.
     * @param token token to return price of
     */
    priceOf(token) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        return token.equals(this.token0) ? this.token0Price : this.token1Price;
    }
    /**
     * Returns the chain ID of the tokens in the pair.
     */
    get chainId() {
        return this.token0.chainId;
    }
    get token0() {
        return this.tokenAmounts[0].currency;
    }
    get token1() {
        return this.tokenAmounts[1].currency;
    }
    get reserve0() {
        return this.tokenAmounts[0];
    }
    get reserve1() {
        return this.tokenAmounts[1];
    }
    get assets() {
        return [this.tokenAmounts[0].currency, this.tokenAmounts[1].currency];
    }
    get reserves() {
        return [this.reserve0, this.reserve1];
    }
    get kLast() {
        return (0, soulswap_math_1.sqrt)(this.reserve0.multiply(this.reserve1).quotient);
    }
    reserveOf(token) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        return token.equals(this.token0) ? this.reserve0 : this.reserve1;
    }
    getOutputAmount(inputAmount) {
        (0, tiny_invariant_1.default)(this.involvesToken(inputAmount.currency), 'TOKEN');
        if (soulswap_math_1.JSBI.equal(this.reserve0.quotient, soulswap_math_1.ZERO) || soulswap_math_1.JSBI.equal(this.reserve1.quotient, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientReservesError();
        }
        const inputReserve = this.reserveOf(inputAmount.currency);
        const outputReserve = this.reserveOf(inputAmount.currency.equals(this.token0) ? this.token1 : this.token0);
        const inputAmountWithFee = soulswap_math_1.JSBI.multiply(inputAmount.quotient, soulswap_math_1.JSBI.subtract(this.MAX_FEE, soulswap_math_1.JSBI.BigInt(this.fee)));
        const numerator = soulswap_math_1.JSBI.multiply(inputAmountWithFee, outputReserve.quotient);
        const denominator = soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.multiply(inputReserve.quotient, this.MAX_FEE), inputAmountWithFee);
        const outputAmount = soulswap_currency_1.Amount.fromRawAmount(inputAmount.currency.equals(this.token0) ? this.token1 : this.token0, soulswap_math_1.JSBI.divide(numerator, denominator));
        if (soulswap_math_1.JSBI.equal(outputAmount.quotient, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientInputAmountError();
        }
        return [
            outputAmount,
            new ConstantProductPool(inputReserve.add(inputAmount), outputReserve.subtract(outputAmount), this.fee, this.twap),
        ];
    }
    getInputAmount(outputAmount) {
        (0, tiny_invariant_1.default)(this.involvesToken(outputAmount.currency), 'TOKEN');
        if (soulswap_math_1.JSBI.equal(this.reserve0.quotient, soulswap_math_1.ZERO) ||
            soulswap_math_1.JSBI.equal(this.reserve1.quotient, soulswap_math_1.ZERO) ||
            soulswap_math_1.JSBI.greaterThanOrEqual(outputAmount.quotient, this.reserveOf(outputAmount.currency).quotient)) {
            throw new errors_1.InsufficientReservesError();
        }
        const outputReserve = this.reserveOf(outputAmount.currency);
        const inputReserve = this.reserveOf(outputAmount.currency.equals(this.token0) ? this.token1 : this.token0);
        const numerator = soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.multiply(inputReserve.quotient, outputAmount.quotient), this.MAX_FEE);
        const denominator = soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.subtract(outputReserve.quotient, outputAmount.quotient), soulswap_math_1.JSBI.subtract(this.MAX_FEE, soulswap_math_1.JSBI.BigInt(this.fee)));
        const inputAmount = soulswap_currency_1.Amount.fromRawAmount(outputAmount.currency.equals(this.token0) ? this.token1 : this.token0, soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.divide(numerator, denominator), soulswap_math_1.ONE));
        return [
            inputAmount,
            new ConstantProductPool(inputReserve.add(inputAmount), outputReserve.subtract(outputAmount), this.fee, this.twap),
        ];
    }
    getNonOptimalMintFee(amount0, amount1, reserve0, reserve1) {
        if (soulswap_math_1.JSBI.equal(reserve0, soulswap_math_1.ZERO) || soulswap_math_1.JSBI.equal(reserve1, soulswap_math_1.ZERO)) {
            return [soulswap_math_1.ZERO, soulswap_math_1.ZERO];
        }
        const amount1Optimal = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amount0, reserve1), reserve0);
        if (soulswap_math_1.JSBI.lessThanOrEqual(amount1Optimal, amount1)) {
            return [
                soulswap_math_1.ZERO,
                soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(this.fee), soulswap_math_1.JSBI.subtract(amount1, amount1Optimal)), soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(2), soulswap_math_1.JSBI.BigInt(10000))),
            ];
        }
        else {
            const amount0Optimal = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amount1, reserve0), reserve1);
            return [
                soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(this.fee), soulswap_math_1.JSBI.subtract(amount0, amount0Optimal)), soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(2), soulswap_math_1.JSBI.BigInt(10000))),
                soulswap_math_1.ZERO,
            ];
        }
    }
    getMintFee(reserve0, reserve1, totalSupply) {
        if (soulswap_math_1.JSBI.notEqual(this.kLast, soulswap_math_1.ZERO)) {
            const computed = (0, soulswap_math_1.sqrt)(soulswap_math_1.JSBI.multiply(reserve0, reserve1));
            if (soulswap_math_1.JSBI.greaterThan(computed, this.kLast)) {
                const liquidity = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.multiply(totalSupply, soulswap_math_1.JSBI.subtract(computed, this.kLast)), soulswap_math_1.JSBI.BigInt(5)), computed), soulswap_math_1.JSBI.BigInt(10000));
                if (soulswap_math_1.JSBI.notEqual(liquidity, soulswap_math_1.ZERO)) {
                    return liquidity;
                }
            }
        }
        return soulswap_math_1.ZERO;
    }
    getLiquidityMinted(totalSupply, tokenAmountA, tokenAmountB) {
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'LIQUIDITY');
        const tokenAmounts = tokenAmountA.currency.sortsBefore(tokenAmountB.currency) // does safety checks
            ? [tokenAmountA, tokenAmountB]
            : [tokenAmountB, tokenAmountA];
        (0, tiny_invariant_1.default)(tokenAmounts[0].currency.equals(this.token0) && tokenAmounts[1].currency.equals(this.token1), 'TOKEN');
        let liquidity;
        // Expected balances after minting
        const balance0 = soulswap_math_1.JSBI.add(tokenAmounts[0].quotient, this.reserve0.quotient);
        const balance1 = soulswap_math_1.JSBI.add(tokenAmounts[1].quotient, this.reserve1.quotient);
        const computed = (0, soulswap_math_1.sqrt)(soulswap_math_1.JSBI.multiply(balance0, balance1));
        if (soulswap_math_1.JSBI.equal(totalSupply.quotient, soulswap_math_1.ZERO)) {
            liquidity = soulswap_math_1.JSBI.subtract(computed, this.minLiquidity);
        }
        else {
            const [fee0, fee1] = this.getNonOptimalMintFee(tokenAmounts[0].quotient, tokenAmounts[1].quotient, this.reserve0.quotient, this.reserve1.quotient);
            const reserve0 = soulswap_math_1.JSBI.add(this.reserve0.quotient, fee0);
            const reserve1 = soulswap_math_1.JSBI.add(this.reserve1.quotient, fee1);
            const k = (0, soulswap_math_1.sqrt)(soulswap_math_1.JSBI.multiply(reserve0, reserve1));
            const mintFee = this.getMintFee(reserve0, reserve1, totalSupply.quotient);
            liquidity = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.subtract(computed, k), soulswap_math_1.JSBI.add(totalSupply.quotient, mintFee)), k);
        }
        if (!soulswap_math_1.JSBI.greaterThan(liquidity, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientInputAmountError();
        }
        return soulswap_currency_1.Amount.fromRawAmount(this.liquidityToken, liquidity);
    }
    getLiquidityValue(token, totalSupply, liquidity) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'TOTAL_SUPPLY');
        (0, tiny_invariant_1.default)(liquidity.currency.equals(this.liquidityToken), 'LIQUIDITY');
        (0, tiny_invariant_1.default)(soulswap_math_1.JSBI.lessThanOrEqual(liquidity.quotient, totalSupply.quotient), 'LIQUIDITY');
        return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserveOf(token).quotient), totalSupply.quotient));
    }
    getAmountOut(amountIn, reserveAmountIn, reserveAmountOut) {
        const amountInWithFee = soulswap_math_1.JSBI.multiply(amountIn, soulswap_math_1.JSBI.subtract(this.MAX_FEE, soulswap_math_1.JSBI.BigInt(this.fee)));
        return soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amountInWithFee, reserveAmountOut), soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.multiply(reserveAmountIn, this.MAX_FEE), amountInWithFee));
    }
    getLiquidityValueSingleToken(token, totalSupply, liquidity) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'TOTAL_SUPPLY');
        (0, tiny_invariant_1.default)(liquidity.currency.equals(this.liquidityToken), 'LIQUIDITY');
        (0, tiny_invariant_1.default)(soulswap_math_1.JSBI.lessThanOrEqual(liquidity.quotient, totalSupply.quotient), 'LIQUIDITY');
        const _totalSupply = soulswap_math_1.JSBI.add(totalSupply.quotient, this.getMintFee(this.reserve0.quotient, this.reserve1.quotient, totalSupply.quotient));
        const amount0 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserve0.quotient), _totalSupply);
        const amount1 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserve1.quotient), _totalSupply);
        if (token === this.token1) {
            return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.add(amount1, this.getAmountOut(amount0, soulswap_math_1.JSBI.subtract(this.reserve0.quotient, amount0), soulswap_math_1.JSBI.subtract(this.reserve1.quotient, amount1))));
        }
        return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.add(amount0, this.getAmountOut(amount1, soulswap_math_1.JSBI.subtract(this.reserve1.quotient, amount1), soulswap_math_1.JSBI.subtract(this.reserve0.quotient, amount0))));
    }
}
exports.ConstantProductPool = ConstantProductPool;
