"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StablePool = void 0;
const all_json_1 = __importDefault(require("@sushiswap/trident/exports/all.json"));
const soulswap_currency_1 = require("soulswap-currency");
const soulswap_math_1 = require("soulswap-math");
const tiny_invariant_1 = __importDefault(require("tiny-invariant"));
const errors_1 = require("../errors");
const computeStablePoolAddress_1 = require("./computeStablePoolAddress");
class StablePool {
    constructor(amountA, amountB, fee, total0, total1) {
        this.swapGasCost = soulswap_math_1.JSBI.BigInt(60000);
        this.minLiquidity = soulswap_math_1.JSBI.BigInt(1000);
        this.MAX_FEE = soulswap_math_1.JSBI.BigInt(10000);
        const tokenAmounts = amountA.currency.sortsBefore(amountB.currency) // does safety checks
            ? [amountA, amountB]
            : [amountB, amountA];
        this.liquidityToken = new soulswap_currency_1.Token({
            chainId: tokenAmounts[0].currency.chainId,
            address: StablePool.getAddress(tokenAmounts[0].currency, tokenAmounts[1].currency, fee),
            decimals: 18,
            symbol: 'SSLP',
            name: 'Sushi Stable LP Token',
        });
        this.fee = fee;
        this.tokenAmounts = tokenAmounts;
        // decimals0 = uint256(10)**(ERC20(_token0).decimals());
        // decimals1 = uint256(10)**(ERC20(_token1).decimals());
        this.decimals0 = soulswap_math_1.JSBI.exponentiate(soulswap_math_1.JSBI.BigInt(10), soulswap_math_1.JSBI.BigInt(this.tokenAmounts[0].currency.decimals));
        this.decimals1 = soulswap_math_1.JSBI.exponentiate(soulswap_math_1.JSBI.BigInt(10), soulswap_math_1.JSBI.BigInt(this.tokenAmounts[1].currency.decimals));
        this.total0 = total0;
        this.total1 = total1;
    }
    static getAddress(tokenA, tokenB, fee) {
        return (0, computeStablePoolAddress_1.computeStablePoolAddress)({
            factoryAddress: all_json_1.default[tokenA.chainId][0].contracts.StablePoolFactory.address,
            tokenA,
            tokenB,
            fee,
        });
    }
    /**
     * Returns true if the token is either token0 or token1
     * @param token to check
     */
    involvesToken(token) {
        return token.equals(this.token0) || token.equals(this.token1);
    }
    /**
     * Returns the current mid price of the pair in terms of token0, i.e. the ratio of reserve1 to reserve0
     */
    get token0Price() {
        const result = this.tokenAmounts[1].divide(this.tokenAmounts[0]);
        return new soulswap_currency_1.Price(this.token0, this.token1, result.denominator, result.numerator);
    }
    /**
     * Returns the current mid price of the pair in terms of token1, i.e. the ratio of reserve0 to reserve1
     */
    get token1Price() {
        const result = this.tokenAmounts[0].divide(this.tokenAmounts[1]);
        return new soulswap_currency_1.Price(this.token1, this.token0, result.denominator, result.numerator);
    }
    /**
     * Return the price of the given token in terms of the other token in the pair.
     * @param token token to return price of
     */
    priceOf(token) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        return token.equals(this.token0) ? this.token0Price : this.token1Price;
    }
    /**
     * Returns the chain ID of the tokens in the pair.
     */
    get chainId() {
        return this.token0.chainId;
    }
    get token0() {
        return this.tokenAmounts[0].currency;
    }
    get token1() {
        return this.tokenAmounts[1].currency;
    }
    get reserve0() {
        return this.tokenAmounts[0];
    }
    get reserve1() {
        return this.tokenAmounts[1];
    }
    get assets() {
        return [this.tokenAmounts[0].currency, this.tokenAmounts[1].currency];
    }
    get reserves() {
        return [this.reserve0, this.reserve1];
    }
    get kLast() {
        return this.computeLiquidity(this.reserve0.quotient, this.reserve1.quotient);
    }
    reserveOf(token) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        return token.equals(this.token0) ? this.reserve0 : this.reserve1;
    }
    _f(x0, y) {
        // (x0 * ((((y * y) / 1e12) * y) / 1e12)) / 1e12
        // +
        // (((((x0 * x0) / 1e12) * x0) / 1e12) * y) / 1e12
        return soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(x0, soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(y, y), soulswap_math_1.JSBI.BigInt(1e12)), y), soulswap_math_1.JSBI.BigInt(1e12))), soulswap_math_1.JSBI.BigInt(1e12)), soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(x0, x0), soulswap_math_1.JSBI.BigInt(1e12)), x0), soulswap_math_1.JSBI.BigInt(1e12)), y), soulswap_math_1.JSBI.BigInt(1e12)));
    }
    _d(x0, y) {
        // (3 * x0 * ((y * y) / 1e12)) / 1e12
        // JSBI.divide(
        //   JSBI.multiply(JSBI.multiply(JSBI.BigInt(3), x0), JSBI.divide(JSBI.multiply(y, y), JSBI.BigInt(1e12))),
        //   JSBI.BigInt(1e12)
        // )
        // +
        // ((((x0 * x0) / 1e12) * x0) / 1e12)
        // JSBI.divide(JSBI.multiply(JSBI.divide(JSBI.multiply(x0, x0), JSBI.BigInt(1e12)), x0), JSBI.BigInt(1e12))
        return soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(3), x0), soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(y, y), soulswap_math_1.JSBI.BigInt(1e12))), soulswap_math_1.JSBI.BigInt(1e12)), soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(x0, x0), soulswap_math_1.JSBI.BigInt(1e12)), x0), soulswap_math_1.JSBI.BigInt(1e12)));
    }
    _get_y(x0, xy, y) {
        for (let i = 0; i < 255; i++) {
            const yPrev = y;
            const k = this._f(x0, y);
            if (soulswap_math_1.JSBI.lessThan(k, xy)) {
                const dy = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.subtract(xy, k), soulswap_math_1.JSBI.BigInt(1e12)), this._d(x0, y));
                y = soulswap_math_1.JSBI.add(y, dy);
            }
            else {
                const dy = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.subtract(k, xy), soulswap_math_1.JSBI.BigInt(1e12)), this._d(x0, y));
                y = soulswap_math_1.JSBI.subtract(y, dy);
            }
            if (soulswap_math_1.JSBI.greaterThan(y, yPrev)) {
                if (soulswap_math_1.JSBI.lessThanOrEqual(soulswap_math_1.JSBI.subtract(y, yPrev), soulswap_math_1.JSBI.BigInt(1))) {
                    return y;
                }
            }
            else {
                if (soulswap_math_1.JSBI.lessThanOrEqual(soulswap_math_1.JSBI.subtract(yPrev, y), soulswap_math_1.JSBI.BigInt(1))) {
                    return y;
                }
            }
        }
        return y;
    }
    _getOutputAmount(inputAmount, reserve0, reserve1) {
        const adjustedReserve0 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(reserve0, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals0);
        const adjustedReserve1 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(reserve1, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals1);
        const feeDeductedAmountIn = soulswap_math_1.JSBI.subtract(inputAmount.quotient, soulswap_math_1.JSBI.multiply(inputAmount.quotient, soulswap_math_1.JSBI.BigInt(this.fee)));
        const xy = this._k(adjustedReserve0, adjustedReserve1);
        // Input is using token0
        if (inputAmount.currency.equals(this.token0)) {
            // uint256 x0 = adjustedReserve0 + ((feeDeductedAmountIn * 1e12) / decimals0);
            // uint256 y = _get_y(x0, xy, adjustedReserve1);
            // dy = adjustedReserve1 - y;
            // dy = (dy * decimals1) / 1e12;
            const x0 = soulswap_math_1.JSBI.add(adjustedReserve0, soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(feeDeductedAmountIn, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals0));
            const y = this._get_y(x0, xy, adjustedReserve1);
            const dy = soulswap_math_1.JSBI.subtract(adjustedReserve1, y);
            return soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(dy, this.decimals1), soulswap_math_1.JSBI.BigInt(1e12));
        }
        else {
            // uint256 x0 = adjustedReserve1 + ((feeDeductedAmountIn * 1e12) / decimals1);
            // uint256 y = _get_y(x0, xy, adjustedReserve0);
            // dy = adjustedReserve0 - y;
            // dy = (dy * decimals0) / 1e12;
            const x0 = soulswap_math_1.JSBI.add(adjustedReserve1, soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(feeDeductedAmountIn, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals1));
            const y = this._get_y(x0, xy, adjustedReserve0);
            const dy = soulswap_math_1.JSBI.subtract(adjustedReserve0, y);
            return soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(dy, this.decimals0), soulswap_math_1.JSBI.BigInt(1e12));
        }
    }
    getOutputAmount(inputShare, inputRebase
    // outputRebase: Rebase
    ) {
        (0, tiny_invariant_1.default)(this.involvesToken(inputShare.currency), 'TOKEN');
        if (soulswap_math_1.JSBI.equal(this.reserve0.quotient, soulswap_math_1.ZERO) || soulswap_math_1.JSBI.equal(this.reserve1.quotient, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientReservesError();
        }
        const inputAmount = inputShare.toAmount(inputRebase, false);
        const outputCurrency = inputAmount.currency.equals(this.token0) ? this.token1 : this.token0;
        const inputReserve = this.reserveOf(inputAmount.currency);
        const outputReserve = this.reserveOf(outputCurrency);
        // uint256 adjustedReserve0 = (_reserve0 * 1e12) / decimals0;
        // uint256 adjustedReserve1 = (_reserve1 * 1e12) / decimals1;
        // uint256 feeDeductedAmountIn = amountIn - (amountIn * swapFee) / MAX_FEE;
        // uint256 xy = _k(adjustedReserve0, adjustedReserve1);
        // if (token0In) {
        //     uint256 x0 = adjustedReserve0 + ((feeDeductedAmountIn * 1e12) / decimals0);
        //     uint256 y = _get_y(x0, xy, adjustedReserve1);
        //     dy = adjustedReserve1 - y;
        //     dy = (dy * decimals1) / 1e12;
        // } else {
        //     uint256 x0 = adjustedReserve1 + ((feeDeductedAmountIn * 1e12) / decimals1);
        //     uint256 y = _get_y(x0, xy, adjustedReserve0);
        //     dy = adjustedReserve0 - y;
        //     dy = (dy * decimals0) / 1e12;
        // }
        // const inputAmountWithFee = JSBI.multiply(inputAmount.quotient, JSBI.subtract(this.MAX_FEE, JSBI.BigInt(this.fee)))
        // const numerator = JSBI.multiply(inputAmountWithFee, outputReserve.quotient)
        // const denominator = JSBI.add(JSBI.multiply(inputReserve.quotient, this.MAX_FEE), inputAmountWithFee)
        // const outputAmount = Amount.fromRawAmount(
        //   inputAmount.currency.equals(this.token0) ? this.token1 : this.token0,
        //   JSBI.divide(numerator, denominator)
        // )
        // TODO: Incomplete...
        const outputAmount = soulswap_currency_1.Amount.fromRawAmount(outputCurrency, this._getOutputAmount(inputAmount, inputReserve.quotient, outputReserve.quotient));
        if (soulswap_math_1.JSBI.equal(outputAmount.quotient, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientInputAmountError();
        }
        return [
            outputAmount,
            new StablePool(inputReserve.add(inputAmount), outputReserve.subtract(outputAmount), this.fee, this.total0, this.total1),
        ];
    }
    // public getInputAmount(outputAmount: Amount<Token>): [Amount<Token>, StablePool] {
    //   invariant(this.involvesToken(outputAmount.currency), 'TOKEN')
    //   if (
    //     JSBI.equal(this.reserve0.quotient, ZERO) ||
    //     JSBI.equal(this.reserve1.quotient, ZERO) ||
    //     JSBI.greaterThanOrEqual(outputAmount.quotient, this.reserveOf(outputAmount.currency).quotient)
    //   ) {
    //     throw new InsufficientReservesError()
    //   }
    //   const outputReserve = this.reserveOf(outputAmount.currency)
    //   const inputReserve = this.reserveOf(outputAmount.currency.equals(this.token0) ? this.token1 : this.token0)
    //   const numerator = JSBI.multiply(JSBI.multiply(inputReserve.quotient, outputAmount.quotient), this.MAX_FEE)
    //   const denominator = JSBI.multiply(
    //     JSBI.subtract(outputReserve.quotient, outputAmount.quotient),
    //     JSBI.subtract(this.MAX_FEE, JSBI.BigInt(this.fee))
    //   )
    //   const inputAmount = Amount.fromRawAmount(
    //     outputAmount.currency.equals(this.token0) ? this.token1 : this.token0,
    //     JSBI.add(JSBI.divide(numerator, denominator), ONE)
    //   )
    //   return [inputAmount, new StablePool(inputReserve.add(inputAmount), outputReserve.subtract(outputAmount), this.fee)]
    // }
    getNonOptimalMintFee(amount0, amount1, reserve0, reserve1) {
        if (soulswap_math_1.JSBI.equal(reserve0, soulswap_math_1.ZERO) || soulswap_math_1.JSBI.equal(reserve1, soulswap_math_1.ZERO)) {
            return [soulswap_math_1.ZERO, soulswap_math_1.ZERO];
        }
        const amount1Optimal = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amount0, reserve1), reserve0);
        if (soulswap_math_1.JSBI.lessThanOrEqual(amount1Optimal, amount1)) {
            return [
                soulswap_math_1.ZERO,
                soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(this.fee), soulswap_math_1.JSBI.subtract(amount1, amount1Optimal)), soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(2), soulswap_math_1.JSBI.BigInt(10000))),
            ];
        }
        else {
            const amount0Optimal = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amount1, reserve0), reserve1);
            return [
                soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(this.fee), soulswap_math_1.JSBI.subtract(amount0, amount0Optimal)), soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.BigInt(2), soulswap_math_1.JSBI.BigInt(10000))),
                soulswap_math_1.ZERO,
            ];
        }
    }
    //   function _k(uint256 x, uint256 y) internal pure returns (uint256) {
    //     uint256 _a = (x * y) / 1e12;
    //     uint256 _b = ((x * x) / 1e12 + (y * y) / 1e12);
    //     return ((_a * _b) / 1e12); // x3y+y3x >= k
    // }
    _k(x, y) {
        const a = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(x, y), soulswap_math_1.JSBI.BigInt(1e12));
        const b = soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(x, x), soulswap_math_1.JSBI.BigInt(1e12)), soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(y, y), soulswap_math_1.JSBI.BigInt(1e12)));
        return soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(a, b), soulswap_math_1.JSBI.BigInt(1e12));
    }
    computeLiquidityFromAdjustedBalances(x, y) {
        return (0, soulswap_math_1.sqrt)((0, soulswap_math_1.sqrt)(this._k(x, y)));
    }
    computeLiquidity(reserve0, reserve1) {
        const adjustedReserve0 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(reserve0, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals0);
        const adjustedReserve1 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(reserve1, soulswap_math_1.JSBI.BigInt(1e12)), this.decimals1);
        return this.computeLiquidityFromAdjustedBalances(adjustedReserve0, adjustedReserve1);
    }
    getMintFee(reserve0, reserve1, totalSupply) {
        if (soulswap_math_1.JSBI.notEqual(this.kLast, soulswap_math_1.ZERO)) {
            const computed = this.computeLiquidity(reserve0, reserve1);
            if (soulswap_math_1.JSBI.greaterThan(computed, this.kLast)) {
                const liquidity = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.multiply(totalSupply, soulswap_math_1.JSBI.subtract(computed, this.kLast)), soulswap_math_1.JSBI.BigInt(5)), computed), soulswap_math_1.JSBI.BigInt(10000));
                if (soulswap_math_1.JSBI.notEqual(liquidity, soulswap_math_1.ZERO)) {
                    return liquidity;
                }
            }
        }
        return soulswap_math_1.ZERO;
    }
    getLiquidityMinted(totalSupply, tokenAmountA, tokenAmountB) {
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'LIQUIDITY');
        const tokenAmounts = tokenAmountA.currency.sortsBefore(tokenAmountB.currency) // does safety checks
            ? [tokenAmountA, tokenAmountB]
            : [tokenAmountB, tokenAmountA];
        (0, tiny_invariant_1.default)(tokenAmounts[0].currency.equals(this.token0) && tokenAmounts[1].currency.equals(this.token1), 'TOKEN');
        let liquidity;
        // Expected balances after minting
        const balance0 = soulswap_math_1.JSBI.add(tokenAmounts[0].quotient, this.reserve0.quotient);
        const balance1 = soulswap_math_1.JSBI.add(tokenAmounts[1].quotient, this.reserve1.quotient);
        const computed = this.computeLiquidity(balance0, balance1);
        if (soulswap_math_1.JSBI.equal(totalSupply.quotient, soulswap_math_1.ZERO)) {
            liquidity = soulswap_math_1.JSBI.subtract(computed, this.minLiquidity);
        }
        else {
            const [fee0, fee1] = this.getNonOptimalMintFee(tokenAmounts[0].quotient, tokenAmounts[1].quotient, this.reserve0.quotient, this.reserve1.quotient);
            const reserve0 = soulswap_math_1.JSBI.add(this.reserve0.quotient, fee0);
            const reserve1 = soulswap_math_1.JSBI.add(this.reserve1.quotient, fee1);
            const k = this.computeLiquidity(reserve0, reserve1);
            const mintFee = this.getMintFee(reserve0, reserve1, totalSupply.quotient);
            liquidity = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(soulswap_math_1.JSBI.subtract(computed, k), soulswap_math_1.JSBI.add(totalSupply.quotient, mintFee)), k);
        }
        if (!soulswap_math_1.JSBI.greaterThan(liquidity, soulswap_math_1.ZERO)) {
            throw new errors_1.InsufficientInputAmountError();
        }
        return soulswap_currency_1.Amount.fromRawAmount(this.liquidityToken, liquidity);
    }
    getLiquidityValue(token, totalSupply, liquidity) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'TOTAL_SUPPLY');
        (0, tiny_invariant_1.default)(liquidity.currency.equals(this.liquidityToken), 'LIQUIDITY');
        (0, tiny_invariant_1.default)(soulswap_math_1.JSBI.lessThanOrEqual(liquidity.quotient, totalSupply.quotient), 'LIQUIDITY');
        return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserveOf(token).quotient), totalSupply.quotient));
    }
    // TODO: unsure if this should change... I guess not.
    getAmountOut(amountIn, reserveAmountIn, reserveAmountOut) {
        const amountInWithFee = soulswap_math_1.JSBI.multiply(amountIn, soulswap_math_1.JSBI.subtract(this.MAX_FEE, soulswap_math_1.JSBI.BigInt(this.fee)));
        return soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(amountInWithFee, reserveAmountOut), soulswap_math_1.JSBI.add(soulswap_math_1.JSBI.multiply(reserveAmountIn, this.MAX_FEE), amountInWithFee));
    }
    getLiquidityValueSingleToken(token, totalSupply, liquidity) {
        (0, tiny_invariant_1.default)(this.involvesToken(token), 'TOKEN');
        (0, tiny_invariant_1.default)(totalSupply.currency.equals(this.liquidityToken), 'TOTAL_SUPPLY');
        (0, tiny_invariant_1.default)(liquidity.currency.equals(this.liquidityToken), 'LIQUIDITY');
        (0, tiny_invariant_1.default)(soulswap_math_1.JSBI.lessThanOrEqual(liquidity.quotient, totalSupply.quotient), 'LIQUIDITY');
        const _totalSupply = soulswap_math_1.JSBI.add(totalSupply.quotient, this.getMintFee(this.reserve0.quotient, this.reserve1.quotient, totalSupply.quotient));
        const amount0 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserve0.quotient), _totalSupply);
        const amount1 = soulswap_math_1.JSBI.divide(soulswap_math_1.JSBI.multiply(liquidity.quotient, this.reserve1.quotient), _totalSupply);
        if (token === this.token1) {
            return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.add(amount1, this.getAmountOut(amount0, soulswap_math_1.JSBI.subtract(this.reserve0.quotient, amount0), soulswap_math_1.JSBI.subtract(this.reserve1.quotient, amount1))));
        }
        return soulswap_currency_1.Amount.fromRawAmount(token, soulswap_math_1.JSBI.add(amount0, this.getAmountOut(amount1, soulswap_math_1.JSBI.subtract(this.reserve1.quotient, amount1), soulswap_math_1.JSBI.subtract(this.reserve0.quotient, amount0))));
    }
}
exports.StablePool = StablePool;
