"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SushiSwapRouter = void 0;
const address_1 = require("@ethersproject/address");
const tiny_invariant_1 = __importDefault(require("tiny-invariant"));
const tiny_warning_1 = __importDefault(require("tiny-warning"));
const Trade_1 = require("./Trade");
// warns if addresses are not checksummed
function validateAndParseAddress(address) {
    try {
        const checksummedAddress = (0, address_1.getAddress)(address);
        (0, tiny_warning_1.default)(address === checksummedAddress, `${address} is not checksummed.`);
        return checksummedAddress;
    }
    catch (error) {
        (0, tiny_invariant_1.default)(false, `${address} is not a valid address.`);
    }
}
const ZERO_HEX = '0x0';
/**
 * Represents the Uniswap V2 Router, and has static methods for helping execute trades.
 */
class SushiSwapRouter {
    /**
     * Cannot be constructed.
     */
    constructor() { }
    /**
     * Produces the on-chain method name to call and the hex encoded parameters to pass as arguments for a given trade.
     * @param trade to produce call parameters for
     * @param options options for the call parameters
     */
    static swapCallParameters(trade, options) {
        const etherIn = trade.inputAmount.currency.isNative;
        const etherOut = trade.outputAmount.currency.isNative;
        // the router does not support both ether in and out
        (0, tiny_invariant_1.default)(!(etherIn && etherOut), 'ETHER_IN_OUT');
        (0, tiny_invariant_1.default)(!('ttl' in options) || options.ttl > 0, 'TTL');
        const to = validateAndParseAddress(options.recipient);
        const amountIn = trade.maximumAmountIn(options.allowedSlippage).toHex();
        const amountOut = trade.minimumAmountOut(options.allowedSlippage).toHex();
        const path = trade.route.legs.reduce((previousValue, currentValue) => [...previousValue, currentValue.tokenTo.address], [trade.route.legs[0].tokenFrom.address]);
        const deadline = 'ttl' in options
            ? `0x${(Math.floor(new Date().getTime() / 1000) + options.ttl).toString(16)}`
            : `0x${options.deadline.toString(16)}`;
        const useFeeOnTransfer = Boolean(options.feeOnTransfer);
        let methodName;
        let args;
        let value;
        switch (trade.tradeType) {
            case Trade_1.TradeType.EXACT_INPUT:
                if (etherIn) {
                    methodName = useFeeOnTransfer ? 'swapExactETHForTokensSupportingFeeOnTransferTokens' : 'swapExactETHForTokens';
                    // (uint amountOutMin, address[] calldata path, address to, uint deadline)
                    args = [amountOut, path, to, deadline];
                    value = amountIn;
                }
                else if (etherOut) {
                    methodName = useFeeOnTransfer ? 'swapExactTokensForETHSupportingFeeOnTransferTokens' : 'swapExactTokensForETH';
                    // (uint amountIn, uint amountOutMin, address[] calldata path, address to, uint deadline)
                    args = [amountIn, amountOut, path, to, deadline];
                    value = ZERO_HEX;
                }
                else {
                    methodName = useFeeOnTransfer
                        ? 'swapExactTokensForTokensSupportingFeeOnTransferTokens'
                        : 'swapExactTokensForTokens';
                    // (uint amountIn, uint amountOutMin, address[] calldata path, address to, uint deadline)
                    args = [amountIn, amountOut, path, to, deadline];
                    value = ZERO_HEX;
                }
                break;
            case Trade_1.TradeType.EXACT_OUTPUT:
                (0, tiny_invariant_1.default)(!useFeeOnTransfer, 'EXACT_OUT_FOT');
                if (etherIn) {
                    methodName = 'swapETHForExactTokens';
                    // (uint amountOut, address[] calldata path, address to, uint deadline)
                    args = [amountOut, path, to, deadline];
                    value = amountIn;
                }
                else if (etherOut) {
                    methodName = 'swapTokensForExactETH';
                    // (uint amountOut, uint amountInMax, address[] calldata path, address to, uint deadline)
                    args = [amountOut, amountIn, path, to, deadline];
                    value = ZERO_HEX;
                }
                else {
                    methodName = 'swapTokensForExactTokens';
                    // (uint amountOut, uint amountInMax, address[] calldata path, address to, uint deadline)
                    args = [amountOut, amountIn, path, to, deadline];
                    value = ZERO_HEX;
                }
                break;
        }
        return {
            methodName,
            args,
            value,
        };
    }
}
exports.SushiSwapRouter = SushiSwapRouter;
