import { Amount, Price, Type as Currency } from 'soulswap-currency';
import { JSBI, Percent } from 'soulswap-math';
import { MultiRoute } from 'soulswap-tines';
import { TradeType as Type } from './TradeType';
import { Version } from './Version';
/**
 * Represents a trade executed against a list of pools.
 * Does not account for slippage, i.e. trades that front run this trade and move the price.
 */
export declare class Trade<TInput extends Currency, TOutput extends Currency, TradeType extends Type, TradeVersion extends Version> {
    /**
     * The route of the trade, i.e. which pools the trade goes through and the input/output currencies.
     */
    readonly route: MultiRoute;
    /**
     * The type of the trade, either exact in or exact out.
     */
    readonly tradeType: TradeType;
    /**
     * The version of the trade, either v1 or v2.
     */
    readonly tradeVersion: TradeVersion;
    /**
     * The input amount for the trade assuming no slippage.
     */
    readonly inputAmount: Amount<TInput>;
    /**
     * The output amount for the trade assuming no slippage.
     */
    readonly outputAmount: Amount<TOutput>;
    /**
     * The input share for the trade assuming no slippage.
     */
    /**
     * The output share for the trade assuming no slippage.
     */
    /**
     * The price expressed in terms of output amount/input amount.
     */
    readonly executionPrice: Price<TInput, TOutput>;
    /**
     * The percent difference between the mid price before the trade and the trade execution price.
     */
    readonly priceImpact: Percent;
    readonly currencyInRebase: {
        base: JSBI;
        elastic: JSBI;
    };
    readonly currencyOutRebase: {
        base: JSBI;
        elastic: JSBI;
    };
    /**
     * Constructs an exact in trade with the given amount in and route
     * @param route route of the exact in trade
     * @param amountIn the amount being passed in
     * @param currencyOut the output currency
     */
    static exactIn<TInput extends Currency, TOutput extends Currency, TVersion extends Version>(route: MultiRoute, amountIn: Amount<TInput>, currencyOut: TOutput, version: TVersion, currencyInRebase?: {
        base: JSBI;
        elastic: JSBI;
    }, currencyOutRebase?: {
        base: JSBI;
        elastic: JSBI;
    }): Trade<TInput, TOutput, Type.EXACT_INPUT, TVersion>;
    /**
     * Constructs an exact out trade with the given amount out and route
     * @param route route of the exact out trade
     * @param amountOut the amount returned by the trade
     * @param currencyIn the output currency
     */
    static exactOut<TInput extends Currency, TOutput extends Currency, TVersion extends Version>(route: MultiRoute, currencyIn: TInput, amountOut: Amount<TOutput>, version: TVersion, currencyInRebase?: {
        base: JSBI;
        elastic: JSBI;
    }, currencyOutRebase?: {
        base: JSBI;
        elastic: JSBI;
    }): Trade<TInput, TOutput, Type.EXACT_OUTPUT, TVersion>;
    constructor(route: MultiRoute, tradeType: TradeType, tradeVersion: TradeVersion, currencyInRebase?: {
        base: JSBI;
        elastic: JSBI;
    }, currencyOutRebase?: {
        base: JSBI;
        elastic: JSBI;
    });
    /**
     * Get the minimum amount that must be received from this trade for the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the execution price of this trade
     */
    minimumAmountOut(slippageTolerance: Percent): Amount<TOutput>;
    /**
     * Get the maximum amount in that can be spent via this trade for the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the execution price of this trade
     */
    maximumAmountIn(slippageTolerance: Percent): Amount<TInput>;
    /**
     * Return the execution price after accounting for slippage tolerance
     * @param slippageTolerance the allowed tolerated slippage
     */
    worstExecutionPrice(slippageTolerance: Percent): Price<TInput, TOutput>;
    isV1(): boolean;
    isV2(): boolean;
    routeType(): 'Single Pool' | 'Single Path' | 'Complex Path' | 'Not Found';
    isNotFound(): boolean;
    isComplex(): boolean;
    isSinglePool(): boolean;
    isSingle(): boolean;
}
//# sourceMappingURL=Trade.d.ts.map